-- Tests on values of the PersonName data type

-- Some tuples with invalid email addresses
-- Should cause an error message to be generated

\echo
\echo -- Checking valid PersonNames ... expect to see name echo'd in canonical form
\echo

select 'Smith, John'::PersonName; 
select 'Smith, John Andrew'::PersonName;
select 'O''Brien, Patrick'::PersonName;
select 'O''brien, Patrick'::PersonName;
select 'Foam-Ocean, David'::PersonName;
select 'Ocean-Foam, Anna-Marie'::PersonName;
select 'O''Brien-O''Byrne, Sean'::PersonName;
select 'A'', O''ch-A''hop'::PersonName;
select 'A''-''-, O''-'''::PersonName;
select 'Xxx, Yyy Zzz Aaa Bbb Ccc Ddd'::PersonName;

\echo
\echo -- Checking invalid PersonNames ... expect to see errors --
\echo

select 'Smith, John  '::PersonName;       -- trailing spaces
select ' Smith, John'::PersonName;        -- leading spaces
select 'Smith,  John'::PersonName;        -- extra space after comma
select 'Smith, john'::PersonName;         -- name begins with lower-case
select 'smith, John'::PersonName;         -- name begins with lower-case
select 'Smith, John A'::PersonName;       -- initials are not names
select 'Smith, John 3rd'::PersonName;     -- no numbers allowed
select 'Able, Baker C#arlie'::PersonName; -- invalid character
select '1234567'::PersonName;             -- not a name
select '-----'::PersonName;               -- not a name

-- should return true

\echo
\echo -- Checking relational operators ... tests below should return True --
\echo

select 'Smith,Jane'::PersonName = 'Smith,Jane'::PersonName;
select 'Smith, Jane'::PersonName = 'Smith, Jane'::PersonName;
select 'Smith,John'::PersonName = 'Smith, John'::PersonName;
select 'Apple, Boy'::PersonName < 'Zebra, Girl'::PersonName;
select 'Apple, Boy'::PersonName < 'Apples, Boy'::PersonName;
select 'Smith,John'::PersonName < 'Smith,John Andrew'::PersonName;
select 'Smith,Jon'::PersonName < 'Smith, Jono'::PersonName;
select 'Zebra,Girl'::PersonName > 'Apple,Boy'::PersonName;
select 'Smith,John'::PersonName > 'Smith,Jane'::PersonName;
select 'Smith,John'::PersonName <= 'Smith,John'::PersonName;
select 'Apple,Girl'::PersonName <= 'Zebra,Boy'::PersonName;
select 'Apple, Boy'::PersonName <= 'Apples, Boy'::PersonName;
select 'Smith,John'::PersonName >= 'Smith,John'::PersonName;
select 'Zebra,Boy'::PersonName >= 'Apple,Girl'::PersonName;
select 'Zebra,Boy'::PersonName <> 'Apple,Girl'::PersonName;
select 'Smith, John'::PersonName <> 'Smith, Jane'::PersonName;
select 'Apple, Boy'::PersonName <> 'Apples, Boy'::PersonName;

-- should return false

\echo
\echo -- Checking relational operators ... tests below should return False --
\echo

select 'Apple, Boy'::PersonName = 'Zebra, Girl'::PersonName;
select 'Smith, Jon'::PersonName = 'Smith, Jono'::PersonName;
select 'O''Brien, Pat'::PersonName = 'O''brien, Pat'::PersonName;
select 'Zebra, Boy'::PersonName < 'Apple, Girl'::PersonName;
select 'Smyth,John'::PersonName < 'Smith,John'::PersonName;
select 'Smith,Jono'::PersonName < 'Smith,Jon'::PersonName;
select 'Apple,Girl'::PersonName > 'Zebra,Boy'::PersonName;
select 'Smith,Jane'::PersonName > 'Smith,John'::PersonName;
select 'Smith,John'::PersonName > 'Smith,John Andrew'::PersonName;
select 'Smyth,John'::PersonName <= 'Smith,John'::PersonName;
select 'Apples,Boy'::PersonName <= 'Apple,Boy'::PersonName;
select 'Smith,Jane Anne'::PersonName >= 'Smith,John Andrew'::PersonName;
select 'Smith,John'::PersonName >= 'Smyth,John'::PersonName;
select 'Smith,John'::PersonName <> 'Smith, John'::PersonName;
select 'Smith, Jane'::PersonName <> 'Smith, Jane'::PersonName;
select 'Smith, Jane'::PersonName <> 'Smith,Jane'::PersonName;

-- These queries assume that the data3 database is loaded

\echo
\echo -- Running some queries --
\echo
\timing on

\echo
\echo -- Count students
\echo -- There should be 10000 Students (assuming data3)
\echo
select count(*) from Students;

\echo
\echo -- Lookup on Students using id and name
\echo
select * from Students where id=3261141; -- exists
-- Barahona Dos Santos,Carla
select * from Students where id=i3108264; -- exists
-- Webb-Martin,Michael Clive
select * from Students where id=3111111; -- doesn't exist

select id from Students
where name = 'Toth, Agnes'::PersonName; -- exists: 3037677
select id from Students
where name = 'Toth,Agnes'::PersonName; -- exists: 3037677
select id from Students
where name = 'Burg,Tracy Lauren Claire Nicola Samantha'::PersonName; -- exists: 3062703
select id from Students
where name = 'Misra,Olla'::PersonName; -- exists: 3079056
select id from Students
where name = 'Mouse, Mickey'::PersoName; -- does not exist

\echo
\echo -- Check ordering for PersonName
\echo
select * from Students order by name limit 10;
select * from Students order by name desc limit 10;

\echo
\echo -- Check grouping based on name (look for repeated names)
\echo
select name, count(*)
from   Students group by name having count(*) > 1
order  by count(*) desc
limit 20;

select given(name), count(*)
from   Students group by given(name) having count(*) > 1
order by count(*) desc
limit 20;

\timing off

-- Check if it can generate and use indexes

\echo
\echo -- Check to see whether hash index works
\echo

create index hash_file on Students using hash (name);

explain analyze
select * from Students
where name = 'Biskin,Diana Hannah'::PersonName;

explain analyze
select * from Students
where name = 'Han, Hao Yao'::PersonName;

drop index hash_file;
